@@ -96,12 +96,6 @@ pub fn functions() -> HashSet<&'static str>
         "asquare",
         "abs",
         "norm",
-        "deg",
-        "degree",
-        "rad",
-        "radian",
-        "grad",
-        "gradian",
         "re",
         "real",
         "im",
@@ -193,8 +187,8 @@ pub fn functions() -> HashSet<&'static str>
         "limit",
         "D",
         "area",
+        "∫",
         "integrate",
-        "length",
         "arclength",
         "roll",
         "erfi",
@@ -204,6 +198,10 @@ pub fn functions() -> HashSet<&'static str>
         "ph",
         "ψ",
         "next",
+        "factorial",
+        "doublefactorial",
+        "subfactorial",
+        "units",
     ]
     .iter()
     .cloned()
