@@ -1,6 +1,6 @@
 use crate::{
-    complex::NumStr::Num, math::do_math, options::set_commands, parse::input_var, Colors, Options,
-    Variable,
+    complex::NumStr::Num, math::do_math, options::set_commands, parse::input_var, Colors, Number,
+    Options, Units, Variable,
 };
 use rug::{float::Constant::Pi, ops::CompleteRound, Complex, Float};
 pub fn get_file_vars(
@@ -66,6 +66,220 @@ pub fn get_file_vars(
         }
     }
 }
+fn ev(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['e', 'V'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("1.602176634e-19").unwrap().complete(prec),
+            Some(Units {
+                meter: 2.0,
+                second: -2.0,
+                kilogram: 1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn ec(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['e', 'C'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("1.602176634e-19").unwrap().complete(prec),
+            Some(Units {
+                ampere: 1.0,
+                second: 1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn kb(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['b', 'o', 'l', 't', 'z', 'm', 'a', 'n', 'n'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("1.380649e-23").unwrap().complete(prec),
+            Some(Units {
+                second: -2.0,
+                meter: 2.0,
+                kilogram: 1.0,
+                kelvin: -1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn me(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['e', 'M'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("9.1093837015e-31").unwrap().complete(prec),
+            Some(Units {
+                kilogram: 1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn mn(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['n', 'M'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("1.67492749804e-27").unwrap().complete(prec),
+            Some(Units {
+                kilogram: 1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn mp(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['p', 'M'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("1.67262192369e-27").unwrap().complete(prec),
+            Some(Units {
+                kilogram: 1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn c(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['c'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("299792458").unwrap().complete(prec),
+            Some(Units {
+                second: -1.0,
+                meter: 1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn g(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['g', 'r', 'a', 'v', 'i', 't', 'y'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("9.80665").unwrap().complete(prec),
+            Some(Units {
+                second: -2.0,
+                meter: 1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn gc(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['G'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("6.67430e-11").unwrap().complete(prec),
+            Some(Units {
+                second: -2.0,
+                meter: 3.0,
+                kilogram: -1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn h(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['p', 'l', 'a', 'n', 'c', 'k'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("6.62607015e-34").unwrap().complete(prec),
+            Some(Units {
+                second: -1.0,
+                meter: 2.0,
+                kilogram: 1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn na(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['N', 'a'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("6.02214076e23").unwrap().complete(prec),
+            Some(Units {
+                mole: -1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn k(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['k', 'e'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("8987551792.3").unwrap().complete(prec),
+            Some(Units {
+                second: -4.0,
+                meter: 3.0,
+                kilogram: 1.0,
+                ampere: -2.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
+fn r(prec: (u32, u32)) -> Variable
+{
+    Variable {
+        name: vec!['R'],
+        parsed: vec![Num(Number::from(
+            Complex::parse("8.31446261815324").unwrap().complete(prec),
+            Some(Units {
+                second: -2.0,
+                meter: 2.0,
+                kilogram: 1.0,
+                kelvin: 1.0,
+                mole: -1.0,
+                ..Units::default()
+            }),
+        ))],
+        unparsed: String::new(),
+        funcvars: Vec::new(),
+    }
+}
 fn get_preset_vars(
     options: Options,
     args: &str,
@@ -73,156 +287,78 @@ fn get_preset_vars(
     blacklist: &mut Vec<String>,
 )
 {
-    if args.contains("ec") && !blacklist.contains(&"ec".to_string())
+    let prec = (options.prec, options.prec);
+    if args.contains("boltzmann") && !blacklist.contains(&"boltzmann".to_string())
     {
-        blacklist.push("ec".to_string());
-        vars.push(Variable {
-            name: vec!['e', 'c'],
-            parsed: vec![Num(Complex::parse("1.602176634e-19")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        blacklist.push("boltzmann".to_string());
+        vars.push(kb(prec));
     }
-    if args.contains("kB") && !blacklist.contains(&"kB".to_string())
+    if args.contains("gravity") && !blacklist.contains(&"gravity".to_string())
     {
-        blacklist.push("kB".to_string());
-        vars.push(Variable {
-            name: vec!['k', 'B'],
-            parsed: vec![Num(Complex::parse("1.380649e-23")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        blacklist.push("gravity".to_string());
+        vars.push(g(prec));
     }
-    if args.contains("me") && !blacklist.contains(&"me".to_string())
+    if args.contains("planck") && !blacklist.contains(&"planck".to_string())
     {
-        blacklist.push("me".to_string());
-        vars.push(Variable {
-            name: vec!['m', 'e'],
-            parsed: vec![Num(Complex::parse("9.1093837015e-31")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        blacklist.push("planck".to_string());
+        vars.push(h(prec));
     }
-    if args.contains("mn") && !blacklist.contains(&"mn".to_string())
+    if args.contains("ke") && !blacklist.contains(&"ke".to_string())
     {
-        blacklist.push("mn".to_string());
-        vars.push(Variable {
-            name: vec!['m', 'n'],
-            parsed: vec![Num(Complex::parse("1.67492749804e-27")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        blacklist.push("ke".to_string());
+        vars.push(k(prec));
     }
-    if args.contains("mp") && !blacklist.contains(&"mp".to_string())
+    if args.contains("eV") && !blacklist.contains(&"eV".to_string())
     {
-        blacklist.push("mp".to_string());
-        vars.push(Variable {
-            name: vec!['m', 'p'],
-            parsed: vec![Num(Complex::parse("1.67262192369e-27")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        blacklist.push("eV".to_string());
+        vars.push(ev(prec));
+    }
+    if args.contains("eC") && !blacklist.contains(&"eC".to_string())
+    {
+        blacklist.push("eC".to_string());
+        vars.push(ec(prec));
+    }
+    if args.contains("eM") && !blacklist.contains(&"eM".to_string())
+    {
+        blacklist.push("eM".to_string());
+        vars.push(me(prec));
+    }
+    if args.contains("nM") && !blacklist.contains(&"nM".to_string())
+    {
+        blacklist.push("nM".to_string());
+        vars.push(mn(prec));
+    }
+    if args.contains("pM") && !blacklist.contains(&"pM".to_string())
+    {
+        blacklist.push("pM".to_string());
+        vars.push(mp(prec));
     }
     if args.contains("Na") && !blacklist.contains(&"Na".to_string())
     {
         blacklist.push("Na".to_string());
-        vars.push(Variable {
-            name: vec!['N', 'a'],
-            parsed: vec![Num(Complex::parse("6.02214076e23")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(na(prec));
     }
     if args.contains('c') && !blacklist.contains(&"c".to_string())
     {
         blacklist.push("c".to_string());
-        vars.push(Variable {
-            name: vec!['c'],
-            parsed: vec![Num(Complex::parse("299792458")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(c(prec));
     }
     if args.contains('G') && !blacklist.contains(&"G".to_string())
     {
         blacklist.push("G".to_string());
-        vars.push(Variable {
-            name: vec!['G'],
-            parsed: vec![Num(Complex::parse("6.67430e-11")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
-    }
-    if args.contains('g') && !blacklist.contains(&"g".to_string())
-    {
-        blacklist.push("g".to_string());
-        vars.push(Variable {
-            name: vec!['g'],
-            parsed: vec![Num(Complex::parse("9.80665")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
-    }
-    if args.contains('h') && !blacklist.contains(&"h".to_string())
-    {
-        blacklist.push("h".to_string());
-        vars.push(Variable {
-            name: vec!['h'],
-            parsed: vec![Num(Complex::parse("6.62607015e-34")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
-    }
-    if args.contains('k') && !blacklist.contains(&"k".to_string())
-    {
-        blacklist.push("k".to_string());
-        vars.push(Variable {
-            name: vec!['k'],
-            parsed: vec![Num(Complex::parse("8.9875517923e9")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(gc(prec));
     }
     if args.contains('R') && !blacklist.contains(&"R".to_string())
     {
         blacklist.push("R".to_string());
-        vars.push(Variable {
-            name: vec!['R'],
-            parsed: vec![Num(Complex::parse("8.31446261815324")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(r(prec));
     }
     {
         let phi1 = args.contains("phi") && !blacklist.contains(&"phi".to_string());
         let phi2 = args.contains('φ') && !blacklist.contains(&"φ".to_string());
         if phi1 || phi2
         {
-            let phi: Float = (1 + Float::with_val(options.prec.0, 5).sqrt()) / 2;
+            let phi: Float = (1 + Float::with_val(options.prec, 5).sqrt()) / 2;
             if phi1
             {
                 blacklist.push("phi".to_string());
@@ -230,7 +366,7 @@ fn get_preset_vars(
                     0,
                     Variable {
                         name: vec!['p', 'h', 'i'],
-                        parsed: vec![Num(phi.clone().into())],
+                        parsed: vec![Num(Number::from(phi.clone().into(), None))],
                         unparsed: String::new(),
                         funcvars: Vec::new(),
                     },
@@ -241,7 +377,7 @@ fn get_preset_vars(
                 blacklist.push("φ".to_string());
                 vars.push(Variable {
                     name: vec!['φ'],
-                    parsed: vec![Num(phi.into())],
+                    parsed: vec![Num(Number::from(phi.into(), None))],
                     unparsed: String::new(),
                     funcvars: Vec::new(),
                 });
@@ -255,7 +391,7 @@ fn get_preset_vars(
         let tau2 = args.contains('τ') && !blacklist.contains(&"τ".to_string());
         if pi1 || pi2 || tau1 || tau2
         {
-            let pi = Float::with_val(options.prec.0, Pi);
+            let pi = Float::with_val(options.prec, Pi);
             if pi1
             {
                 blacklist.push("pi".to_string());
@@ -263,7 +399,7 @@ fn get_preset_vars(
                     vars.iter().position(|c| c.name.len() != 3).unwrap_or(0),
                     Variable {
                         name: vec!['p', 'i'],
-                        parsed: vec![Num(pi.clone().into())],
+                        parsed: vec![Num(Number::from(pi.clone().into(), None))],
                         unparsed: String::new(),
                         funcvars: Vec::new(),
                     },
@@ -274,7 +410,7 @@ fn get_preset_vars(
                 blacklist.push("π".to_string());
                 vars.push(Variable {
                     name: vec!['π'],
-                    parsed: vec![Num(pi.clone().into())],
+                    parsed: vec![Num(Number::from(pi.clone().into(), None))],
                     unparsed: String::new(),
                     funcvars: Vec::new(),
                 });
@@ -289,7 +425,7 @@ fn get_preset_vars(
                         0,
                         Variable {
                             name: vec!['t', 'a', 'u'],
-                            parsed: vec![Num(tau.clone().into())],
+                            parsed: vec![Num(Number::from(tau.clone().into(), None))],
                             unparsed: String::new(),
                             funcvars: Vec::new(),
                         },
@@ -300,7 +436,7 @@ fn get_preset_vars(
                     blacklist.push("τ".to_string());
                     vars.push(Variable {
                         name: vec!['τ'],
-                        parsed: vec![Num(tau.into())],
+                        parsed: vec![Num(Number::from(tau.into(), None))],
                         unparsed: String::new(),
                         funcvars: Vec::new(),
                     });
@@ -311,10 +447,10 @@ fn get_preset_vars(
     if args.contains('e') && !blacklist.contains(&"e".to_string())
     {
         blacklist.push("e".to_string());
-        let e = Float::with_val(options.prec.0, 1).exp();
+        let e = Float::with_val(options.prec, 1).exp();
         vars.push(Variable {
             name: vec!['e'],
-            parsed: vec![Num(e.into())],
+            parsed: vec![Num(Number::from(e.into(), None))],
             unparsed: String::new(),
             funcvars: Vec::new(),
         });
@@ -326,151 +462,72 @@ pub fn get_cli_vars(options: Options, args: String, vars: &mut Vec<Variable>)
     {
         return;
     }
-    if args.contains("ec")
+    let prec = (options.prec, options.prec);
+    if args.contains("boltzmann")
     {
-        vars.push(Variable {
-            name: vec!['e', 'c'],
-            parsed: vec![Num(Complex::parse("1.602176634e-19")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(kb(prec));
     }
-    if args.contains("kB")
+    if args.contains("gravity")
     {
-        vars.push(Variable {
-            name: vec!['k', 'B'],
-            parsed: vec![Num(Complex::parse("1.380649e-23")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(g(prec));
     }
-    if args.contains("me")
+    if args.contains("planck")
     {
-        vars.push(Variable {
-            name: vec!['m', 'e'],
-            parsed: vec![Num(Complex::parse("9.1093837015e-31")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(h(prec));
     }
-    if args.contains("mn")
+    if args.contains("ke")
     {
-        vars.push(Variable {
-            name: vec!['m', 'n'],
-            parsed: vec![Num(Complex::parse("1.67492749804e-27")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(k(prec));
     }
-    if args.contains("mp")
+    if args.contains("eV")
     {
-        vars.push(Variable {
-            name: vec!['m', 'p'],
-            parsed: vec![Num(Complex::parse("1.67262192369e-27")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(ev(prec));
+    }
+    if args.contains("eC")
+    {
+        vars.push(ec(prec));
+    }
+    if args.contains("eM")
+    {
+        vars.push(me(prec));
+    }
+    if args.contains("nM")
+    {
+        vars.push(mn(prec));
+    }
+    if args.contains("pM")
+    {
+        vars.push(mp(prec));
     }
     if args.contains("Na")
     {
-        vars.push(Variable {
-            name: vec!['N', 'a'],
-            parsed: vec![Num(Complex::parse("6.02214076e23")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(na(prec));
     }
     if args.contains('c')
     {
-        vars.push(Variable {
-            name: vec!['c'],
-            parsed: vec![Num(Complex::parse("299792458")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(c(prec));
     }
     if args.contains('G')
     {
-        vars.push(Variable {
-            name: vec!['G'],
-            parsed: vec![Num(Complex::parse("6.67430e-11")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
-    }
-    if args.contains('g')
-    {
-        vars.push(Variable {
-            name: vec!['g'],
-            parsed: vec![Num(Complex::parse("9.80665")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
-    }
-    if args.contains('h')
-    {
-        vars.push(Variable {
-            name: vec!['h'],
-            parsed: vec![Num(Complex::parse("6.62607015e-34")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
-    }
-    if args.contains('k')
-    {
-        vars.push(Variable {
-            name: vec!['k'],
-            parsed: vec![Num(Complex::parse("8.9875517923e9")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(gc(prec));
     }
     if args.contains('R')
     {
-        vars.push(Variable {
-            name: vec!['R'],
-            parsed: vec![Num(Complex::parse("8.31446261815324")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        });
+        vars.push(r(prec));
     }
     {
         let phi1 = args.contains("phi");
         let phi2 = args.contains('φ');
         if phi1 || phi2
         {
-            let phi: Float = (1 + Float::with_val(options.prec.0, 5).sqrt()) / 2;
+            let phi: Float = (1 + Float::with_val(options.prec, 5).sqrt()) / 2;
             if phi1
             {
                 vars.insert(
                     0,
                     Variable {
                         name: vec!['p', 'h', 'i'],
-                        parsed: vec![Num(phi.clone().into())],
+                        parsed: vec![Num(Number::from(phi.clone().into(), None))],
                         unparsed: String::new(),
                         funcvars: Vec::new(),
                     },
@@ -480,7 +537,7 @@ pub fn get_cli_vars(options: Options, args: String, vars: &mut Vec<Variable>)
             {
                 vars.push(Variable {
                     name: vec!['φ'],
-                    parsed: vec![Num(phi.into())],
+                    parsed: vec![Num(Number::from(phi.into(), None))],
                     unparsed: String::new(),
                     funcvars: Vec::new(),
                 });
@@ -494,14 +551,14 @@ pub fn get_cli_vars(options: Options, args: String, vars: &mut Vec<Variable>)
         let tau2 = args.contains('τ');
         if pi1 || pi2 || tau1 || tau2
         {
-            let pi = Float::with_val(options.prec.0, Pi);
+            let pi = Float::with_val(options.prec, Pi);
             if pi1
             {
                 vars.insert(
                     vars.iter().position(|c| c.name.len() != 3).unwrap_or(0),
                     Variable {
                         name: vec!['p', 'i'],
-                        parsed: vec![Num(pi.clone().into())],
+                        parsed: vec![Num(Number::from(pi.clone().into(), None))],
                         unparsed: String::new(),
                         funcvars: Vec::new(),
                     },
@@ -511,7 +568,7 @@ pub fn get_cli_vars(options: Options, args: String, vars: &mut Vec<Variable>)
             {
                 vars.push(Variable {
                     name: vec!['π'],
-                    parsed: vec![Num(pi.clone().into())],
+                    parsed: vec![Num(Number::from(pi.clone().into(), None))],
                     unparsed: String::new(),
                     funcvars: Vec::new(),
                 });
@@ -525,7 +582,7 @@ pub fn get_cli_vars(options: Options, args: String, vars: &mut Vec<Variable>)
                         0,
                         Variable {
                             name: vec!['t', 'a', 'u'],
-                            parsed: vec![Num(tau.clone().into())],
+                            parsed: vec![Num(Number::from(tau.clone().into(), None))],
                             unparsed: String::new(),
                             funcvars: Vec::new(),
                         },
@@ -535,7 +592,7 @@ pub fn get_cli_vars(options: Options, args: String, vars: &mut Vec<Variable>)
                 {
                     vars.push(Variable {
                         name: vec!['τ'],
-                        parsed: vec![Num(tau.into())],
+                        parsed: vec![Num(Number::from(tau.into(), None))],
                         unparsed: String::new(),
                         funcvars: Vec::new(),
                     });
@@ -545,10 +602,10 @@ pub fn get_cli_vars(options: Options, args: String, vars: &mut Vec<Variable>)
     }
     if args.contains('e')
     {
-        let e = Float::with_val(options.prec.0, 1).exp();
+        let e = Float::with_val(options.prec, 1).exp();
         vars.push(Variable {
             name: vec!['e'],
-            parsed: vec![Num(e.into())],
+            parsed: vec![Num(Number::from(e.into(), None))],
             unparsed: String::new(),
             funcvars: Vec::new(),
         });
@@ -556,146 +613,64 @@ pub fn get_cli_vars(options: Options, args: String, vars: &mut Vec<Variable>)
 }
 pub fn get_vars(options: Options) -> Vec<Variable>
 {
-    let pi = Float::with_val(options.prec.0, Pi);
+    let prec = (options.prec, options.prec);
+    let pi = Float::with_val(options.prec, Pi);
     let tau: Float = pi.clone() * 2;
-    let phi: Float = (1 + Float::with_val(options.prec.0, 5).sqrt()) / 2;
-    let e = Float::with_val(options.prec.0, 1).exp();
+    let phi: Float = (1 + Float::with_val(options.prec, 5).sqrt()) / 2;
+    let e = Float::with_val(options.prec, 1).exp();
     vec![
+        kb(prec),
+        g(prec),
+        h(prec),
         Variable {
             name: vec!['p', 'h', 'i'],
-            parsed: vec![Num(phi.clone().into())],
+            parsed: vec![Num(Number::from(phi.clone().into(), None))],
             unparsed: String::new(),
             funcvars: Vec::new(),
         },
         Variable {
             name: vec!['t', 'a', 'u'],
-            parsed: vec![Num(tau.clone().into())],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['e', 'c'],
-            parsed: vec![Num(Complex::parse("1.602176634e-19")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['k', 'B'],
-            parsed: vec![Num(Complex::parse("1.380649e-23")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['m', 'e'],
-            parsed: vec![Num(Complex::parse("9.1093837015e-31")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['m', 'n'],
-            parsed: vec![Num(Complex::parse("1.67492749804e-27")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['m', 'p'],
-            parsed: vec![Num(Complex::parse("1.67262192369e-27")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['N', 'a'],
-            parsed: vec![Num(Complex::parse("6.02214076e23")
-                .unwrap()
-                .complete(options.prec))],
+            parsed: vec![Num(Number::from(tau.clone().into(), None))],
             unparsed: String::new(),
             funcvars: Vec::new(),
         },
+        ec(prec),
+        ev(prec),
+        me(prec),
+        mn(prec),
+        mp(prec),
+        k(prec),
+        na(prec),
         Variable {
             name: vec!['p', 'i'],
-            parsed: vec![Num(pi.clone().into())],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['c'],
-            parsed: vec![Num(Complex::parse("299792458")
-                .unwrap()
-                .complete(options.prec))],
+            parsed: vec![Num(Number::from(pi.clone().into(), None))],
             unparsed: String::new(),
             funcvars: Vec::new(),
         },
+        c(prec),
         Variable {
             name: vec!['e'],
-            parsed: vec![Num(e.into())],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['G'],
-            parsed: vec![Num(Complex::parse("6.67430e-11")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['g'],
-            parsed: vec![Num(Complex::parse("9.80665")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['h'],
-            parsed: vec![Num(Complex::parse("6.62607015e-34")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['k'],
-            parsed: vec![Num(Complex::parse("8.9875517923e9")
-                .unwrap()
-                .complete(options.prec))],
-            unparsed: String::new(),
-            funcvars: Vec::new(),
-        },
-        Variable {
-            name: vec!['R'],
-            parsed: vec![Num(Complex::parse("8.31446261815324")
-                .unwrap()
-                .complete(options.prec))],
+            parsed: vec![Num(Number::from(e.into(), None))],
             unparsed: String::new(),
             funcvars: Vec::new(),
         },
+        gc(prec),
+        r(prec),
         Variable {
             name: vec!['φ'],
-            parsed: vec![Num(phi.into())],
+            parsed: vec![Num(Number::from(phi.into(), None))],
             unparsed: String::new(),
             funcvars: Vec::new(),
         },
         Variable {
             name: vec!['π'],
-            parsed: vec![Num(pi.into())],
+            parsed: vec![Num(Number::from(pi.into(), None))],
             unparsed: String::new(),
             funcvars: Vec::new(),
         },
         Variable {
             name: vec!['τ'],
-            parsed: vec![Num(tau.into())],
+            parsed: vec![Num(Number::from(tau.into(), None))],
             unparsed: String::new(),
             funcvars: Vec::new(),
         },
