@@ -11,6 +11,7 @@ mod parse;
 mod print;
 #[cfg(test)]
 mod tests;
+mod units;
 use crate::{
     complex::NumStr,
     graph::graph,
@@ -24,11 +25,13 @@ use crate::{
     parse::input_var,
     print::{print_answer, print_concurrent},
     AngleType::Radians,
+    Notation::Normal,
 };
 use crossterm::{
     cursor::{DisableBlinking, EnableBlinking},
     execute, terminal,
 };
+use rug::Complex;
 use std::{
     env::{args, var},
     fs::{File, OpenOptions},
@@ -36,17 +39,20 @@ use std::{
     thread::JoinHandle,
     time::Instant,
 };
-//derivitives and integrals
-//surface area of a 3d curve
 //rpn
-//matrix exponentiation
-//do as much as you can before graphing, make '=' at end show unsimplified, and no = show simplififed
-//stop vectors or matrixes disipearing graphically by making them have points instead of lines
-//maybe nth area, make -nth go into the other function
-//allow setting x and y window dimensions of gnuplot, defaulting to what happens now
-//setting to disable default vars by default, in config
 //domain coloring
-//support units properly, add a part to the Num struct where it just stores the unit which then can be dealt with in complex or smth
+//matrix exponentiation
+//surface area of a 3d curve
+//maybe nth area, make -nth go into the other function
+//stop vectors or matrixes disipearing graphically by making them have points instead of lines
+//do as much as you can before graphing, make '=' at end show unsimplified, and no = show simplififed
+//allow setting x and y window dimensions of gnuplot, defaulting to what happens now
+//infinitely long greater/lesser than chains
+//have unit powers be fractional, maybe 2 usizes
+//add option to print units in simplified form instead of just in base si units, like N instead of kg m s^-2
+//compress units if makes sense, ie m s^-2 -> m/s^2
+//allow changing the default unit for all, like angles
+//maybe add exchange rates via something like https://www.floatrates.com/daily/usd.json
 #[derive(Clone)]
 pub struct Variable
 {
@@ -55,6 +61,25 @@ pub struct Variable
     unparsed: String,
     funcvars: Vec<(String, Vec<NumStr>)>,
 }
+#[derive(Clone, PartialEq, Copy)]
+pub struct Units
+{
+    second: f64,
+    meter: f64,
+    kilogram: f64,
+    ampere: f64,
+    kelvin: f64,
+    mole: f64,
+    candela: f64,
+    angle: f64,
+    byte: f64,
+}
+#[derive(Clone, PartialEq)]
+pub struct Number
+{
+    number: Complex,
+    units: Option<Units>,
+}
 #[derive(Clone)]
 pub struct Colors
 {
@@ -62,6 +87,7 @@ pub struct Colors
     prompt: String,
     imag: String,
     sci: String,
+    units: String,
     brackets: Vec<String>,
     re1col: String,
     re2col: String,
@@ -86,6 +112,7 @@ impl Default for Colors
             prompt: "\x1b[94m".to_string(),
             imag: "\x1b[93m".to_string(),
             sci: "\x1b[92m".to_string(),
+            units: "\x1b[96m".to_string(),
             brackets: vec![
                 "\x1b[91m".to_string(),
                 "\x1b[92m".to_string(),
@@ -117,15 +144,22 @@ pub enum AngleType
     Degrees,
     Gradians,
 }
+#[derive(Copy, Clone, PartialEq)]
+pub enum Notation
+{
+    Normal,
+    Scientific,
+    LargeEngineering,
+    SmallEngineering,
+}
 #[derive(Clone, Copy)]
 pub struct Options
 {
-    sci: bool,
-    deg: AngleType,
-    base: usize,
+    notation: Notation,
+    angle: AngleType,
+    base: (i32, i32),
     ticks: f64,
     onaxis: bool,
-    tau: bool,
     polar: bool,
     frac: bool,
     real_time_output: bool,
@@ -133,9 +167,8 @@ pub struct Options
     color: bool,
     prompt: bool,
     comma: bool,
-    prec: (u32, u32),
-    graph_prec: (u32, u32),
-    frac_iter: usize,
+    prec: u32,
+    graph_prec: u32,
     xr: (f64, f64),
     yr: (f64, f64),
     zr: (f64, f64),
@@ -149,30 +182,29 @@ pub struct Options
     multi: bool,
     tabbed: bool,
     allow_vars: bool,
-    small_e: bool,
     debug: bool,
     depth: bool,
     flat: bool,
     graph: bool,
     slowcheck: u128,
-    var_multiply: bool,
     interactive: bool,
     surface: bool,
     scale_graph: bool,
     stay_interactive: bool,
     graph_cli: bool,
+    units: bool,
+    window_size: (usize, usize),
 }
 impl Default for Options
 {
     fn default() -> Self
     {
         Self {
-            sci: false,
-            deg: Radians,
-            base: 10,
+            notation: Normal,
+            angle: Radians,
+            base: (10, 10),
             ticks: 20.0,
             onaxis: true,
-            tau: false,
             polar: false,
             frac: true,
             real_time_output: true,
@@ -180,9 +212,8 @@ impl Default for Options
             color: true,
             prompt: true,
             comma: false,
-            prec: (512, 512),
-            graph_prec: (128, 128),
-            frac_iter: 50,
+            prec: 512,
+            graph_prec: 128,
             xr: (-10.0, 10.0),
             yr: (-10.0, 10.0),
             zr: (-10.0, 10.0),
@@ -196,18 +227,18 @@ impl Default for Options
             multi: true,
             tabbed: false,
             allow_vars: true,
-            small_e: false,
             debug: false,
             depth: false,
             flat: false,
             graph: true,
             slowcheck: 300,
-            var_multiply: false,
             interactive: true,
             surface: false,
-            scale_graph: true,
+            scale_graph: false,
             stay_interactive: false,
             graph_cli: false,
+            units: true,
+            window_size: (0, 0),
         }
     }
 }
@@ -216,6 +247,7 @@ fn main()
     let mut colors = Colors::default();
     let mut options = Options::default();
     let mut args = args().collect::<Vec<String>>();
+    let mut default = false;
     {
         #[cfg(unix)]
         let file_path = &(var("HOME").unwrap() + "/.config/kalc.config");
@@ -229,10 +261,20 @@ fn main()
             println!("{}", s);
             std::process::exit(1);
         }
-        if let Err(s) = arg_opts(&mut options, &mut colors, &mut args)
+        match arg_opts(&mut options, &mut colors, &mut args)
         {
-            println!("{}", s);
-            std::process::exit(1);
+            Ok(s) =>
+            {
+                if s
+                {
+                    default = true
+                }
+            }
+            Err(s) =>
+            {
+                println!("{}", s);
+                std::process::exit(1);
+            }
         }
     }
     if !stdin().is_terminal()
@@ -290,13 +332,15 @@ fn main()
     {
         Vec::new()
     };
+    let base = options.base;
     if !options.interactive && options.allow_vars && !options.stay_interactive
     {
         get_cli_vars(options, args.concat(), &mut vars)
     }
     {
-        if options.allow_vars
+        if options.allow_vars && !default
         {
+            options.base = (10, 10);
             if let Ok(file) = File::open(file_path)
             {
                 let lines = BufReader::new(file)
@@ -411,6 +455,7 @@ fn main()
             }
         }
     }
+    options.base = base;
     let (mut file, mut unmod_lines) = if options.interactive || options.stay_interactive
     {
         #[cfg(unix)]
@@ -571,7 +616,7 @@ fn main()
                             end = input.len()
                         }
                         if (!options.real_time_output || long || (slow && !firstslow))
-                            && !input.is_empty()
+                            && (!input.is_empty() && !input.starts_with(&['#']))
                             && c != '\x14'
                             && c != '\x06'
                         {
@@ -587,7 +632,7 @@ fn main()
                             );
                             slow = watch.elapsed().as_millis() > options.slowcheck;
                         }
-                        if !input.is_empty()
+                        if !input.is_empty() && !input.starts_with(&['#'])
                         {
                             let num = frac + if !graphable && !varcheck { 1 } else { 0 };
                             if num != 0
@@ -682,6 +727,10 @@ fn main()
                         {
                             end = input.len()
                         }
+                        if start >= end
+                        {
+                            start = end;
+                        }
                         if i == lines.len()
                         {
                             current.clone_from(&input);
@@ -1352,6 +1401,7 @@ fn main()
                                 {
                                     if hit
                                     {
+                                        hit = false;
                                         placement = i + 1;
                                         break;
                                     }
@@ -1361,10 +1411,16 @@ fn main()
                                     hit = true;
                                 }
                             }
+                            if hit
+                            {
+                                placement = 1;
+                            }
                             if placement <= start
                             {
-                                end -= start - placement;
-                                start = start - (start - placement);
+                                end = placement
+                                    + (get_terminal_dimensions().0
+                                        - if options.prompt { 3 } else { 1 });
+                                start = placement;
                                 clearln(&input, start, end, options, &colors);
                                 if end - placement != 0
                                 {
@@ -1395,6 +1451,7 @@ fn main()
                                 {
                                     if hit
                                     {
+                                        hit = false;
                                         placement += i + 1;
                                         break;
                                     }
@@ -1404,10 +1461,16 @@ fn main()
                                     hit = true;
                                 }
                             }
+                            if hit
+                            {
+                                placement = input.len() - 1;
+                            }
                             if placement >= end
                             {
-                                start += placement - end;
-                                end = end + (placement - end);
+                                start = placement
+                                    - (get_terminal_dimensions().0
+                                        - if options.prompt { 3 } else { 1 });
+                                end = placement;
                                 clearln(&input, start, end, options, &colors)
                             }
                             else if placement == s
@@ -1507,11 +1570,7 @@ fn main()
                 &colors,
                 &mut vars,
                 &lines,
-                &input
-                    .clone()
-                    .into_iter()
-                    .filter(|&c| !c.is_whitespace())
-                    .collect::<Vec<char>>(),
+                &input,
                 &mut stdout,
             );
             if !varcheck
@@ -1532,11 +1591,7 @@ fn main()
                 input
                     .iter()
                     .collect::<String>()
-                    .replace("small_e", "smalle")
-                    .replace("frac_iter", "fraciter")
-                    .replace('_', &format!("({})", last.iter().collect::<String>()))
-                    .replace("smalle", "small_e")
-                    .replace("fraciter", "frac_iter"),
+                    .replace('_', &format!("({})", last.iter().collect::<String>())),
                 file.as_mut().unwrap(),
                 unmod_lines.as_mut().unwrap(),
                 slow,
@@ -1549,7 +1604,7 @@ fn main()
             {
                 if !s.is_empty()
                 {
-                    print!("\x1b[G\x1b[K{}\n\x1b[G{}", s, prompt(options, &colors));
+                    print!("\x1b[G\x1b[K{}\x1b[G\n{}", s, prompt(options, &colors));
                 }
                 else
                 {
