@@ -8,7 +8,7 @@ use crate::{
     print::get_output,
     Colors, Options,
 };
-use crossterm::event::{read, Event, KeyCode, KeyEvent, KeyModifiers};
+use crossterm::event::{read, Event, KeyCode, KeyEvent, KeyEventKind, KeyModifiers};
 #[cfg(unix)]
 use libc::{ioctl, winsize, STDOUT_FILENO, TIOCGWINSZ};
 use std::{fs::File, io::Write};
@@ -45,33 +45,29 @@ pub fn get_terminal_dimensions_pixel() -> (usize, usize)
     }
 }
 #[cfg(not(unix))]
-pub fn get_terminal_width() -> usize
+pub fn get_terminal_dimensions() -> (usize, usize)
 {
-    if let Some((width, _)) = dimensions()
+    if let Some((width, height)) = dimensions()
     {
-        width
+        (width, height)
     }
     else
     {
-        80
+        (80, 80)
     }
 }
-#[cfg(not(unix))]
-pub fn get_terminal_height() -> usize
+pub fn digraph(char: Option<char>) -> char
 {
-    if let Some((_, height)) = dimensions()
+    match if let Some(c) = char
     {
-        height
+        c
     }
     else
     {
-        80
+        read_single_char()
     }
-}
-pub fn digraph() -> char
-{
-    match read_single_char()
     {
+        ';' => '°',
         'a' => 'α',
         'A' => 'Α',
         'b' => 'β',
@@ -153,7 +149,7 @@ pub fn convert(c: &char) -> char
 {
     let valid_chars = [
         '+', '^', '(', ')', '.', '=', ',', '#', '|', '&', '!', '%', '_', '<', '>', ' ', '[', ']',
-        '{', '}', '√', '∛', '⁻', 'ⁱ', '`', '±', '∞', ';', ':', '\'',
+        '{', '}', '√', '∛', '⁻', 'ⁱ', '`', '±', '∞', ';', ':', '\'', '⌊', '⌈', '⌉', '⌋', '∫', '°',
     ];
     match c
     {
@@ -173,40 +169,51 @@ pub fn read_single_char() -> char
     }
     {
         Event::Key(KeyEvent {
-            code, modifiers, ..
+            code,
+            modifiers,
+            kind,
+            ..
         }) =>
         {
-            match (code, modifiers)
+            if kind == KeyEventKind::Press
             {
-                (KeyCode::Char('c'), KeyModifiers::CONTROL) => '\x14',
-                (KeyCode::Home, KeyModifiers::NONE)
-                | (KeyCode::Char('a'), KeyModifiers::CONTROL) => '\x10',
-                (KeyCode::End, KeyModifiers::NONE)
-                | (KeyCode::Char('e'), KeyModifiers::CONTROL) => '\x11',
-                (KeyCode::Left, KeyModifiers::CONTROL)
-                | (KeyCode::Char('b'), KeyModifiers::ALT) => '\x12',
-                (KeyCode::Right, KeyModifiers::CONTROL)
-                | (KeyCode::Char('f'), KeyModifiers::ALT) => '\x13',
-                (KeyCode::Char('l'), KeyModifiers::CONTROL) => '\x15',
-                (KeyCode::Char('t'), KeyModifiers::CONTROL) => '\x16',
-                (KeyCode::Char('u'), KeyModifiers::CONTROL) => '\x18',
-                (KeyCode::Char('k'), KeyModifiers::CONTROL) => '\x19',
-                (KeyCode::Char('y'), KeyModifiers::CONTROL) => '\x17',
-                (KeyCode::Char('d'), KeyModifiers::CONTROL) => '\x06',
-                (KeyCode::Char('p'), KeyModifiers::CONTROL) => '\x05',
-                (KeyCode::Char('n'), KeyModifiers::CONTROL) => '\x04',
-                (KeyCode::Char(c), KeyModifiers::NONE | KeyModifiers::SHIFT) => convert(&c),
-                (KeyCode::Esc, _) => digraph(),
-                (KeyCode::Enter, KeyModifiers::NONE) => '\n',
-                (KeyCode::Enter, KeyModifiers::ALT) => '\x09',
-                (KeyCode::Char('h'), KeyModifiers::CONTROL) => '\x03',
-                (KeyCode::Backspace, KeyModifiers::NONE) => '\x08',
-                (KeyCode::Delete, KeyModifiers::NONE) => '\x7F',
-                (KeyCode::Left, KeyModifiers::NONE) => '\x1B',
-                (KeyCode::Right, KeyModifiers::NONE) => '\x1C',
-                (KeyCode::Up, KeyModifiers::NONE) => '\x1D',
-                (KeyCode::Down, KeyModifiers::NONE) => '\x1E',
-                _ => '\0',
+                match (code, modifiers)
+                {
+                    (KeyCode::Char('c'), KeyModifiers::CONTROL) => '\x14',
+                    (KeyCode::Home, KeyModifiers::NONE)
+                    | (KeyCode::Char('a'), KeyModifiers::CONTROL) => '\x10',
+                    (KeyCode::End, KeyModifiers::NONE)
+                    | (KeyCode::Char('e'), KeyModifiers::CONTROL) => '\x11',
+                    (KeyCode::Left, KeyModifiers::CONTROL)
+                    | (KeyCode::Char('b'), KeyModifiers::ALT) => '\x12',
+                    (KeyCode::Right, KeyModifiers::CONTROL)
+                    | (KeyCode::Char('f'), KeyModifiers::ALT) => '\x13',
+                    (KeyCode::Char('l'), KeyModifiers::CONTROL) => '\x15',
+                    (KeyCode::Char('t'), KeyModifiers::CONTROL) => '\x16',
+                    (KeyCode::Char('u'), KeyModifiers::CONTROL) => '\x18',
+                    (KeyCode::Char('k'), KeyModifiers::CONTROL) => '\x19',
+                    (KeyCode::Char('y'), KeyModifiers::CONTROL) => '\x17',
+                    (KeyCode::Char('d'), KeyModifiers::CONTROL) => '\x06',
+                    (KeyCode::Char('p'), KeyModifiers::CONTROL) => '\x05',
+                    (KeyCode::Char('n'), KeyModifiers::CONTROL) => '\x04',
+                    (KeyCode::Char(c), KeyModifiers::NONE | KeyModifiers::SHIFT) => convert(&c),
+                    (KeyCode::Char(c), KeyModifiers::ALT) => digraph(Some(c)),
+                    (KeyCode::Esc, _) => digraph(None),
+                    (KeyCode::Enter, KeyModifiers::NONE) => '\n',
+                    (KeyCode::Enter, KeyModifiers::ALT) => '\x09',
+                    (KeyCode::Char('h'), KeyModifiers::CONTROL) => '\x03',
+                    (KeyCode::Backspace, KeyModifiers::NONE) => '\x08',
+                    (KeyCode::Delete, KeyModifiers::NONE) => '\x7F',
+                    (KeyCode::Left, KeyModifiers::NONE) => '\x1B',
+                    (KeyCode::Right, KeyModifiers::NONE) => '\x1C',
+                    (KeyCode::Up, KeyModifiers::NONE) => '\x1D',
+                    (KeyCode::Down, KeyModifiers::NONE) => '\x1E',
+                    _ => '\0',
+                }
+            }
+            else
+            {
+                '\0'
             }
         }
         _ => '\0',
@@ -245,7 +252,7 @@ pub fn no_col(input: &str, color: bool) -> Vec<char>
 }
 pub fn write(mut input: String, file: &mut File, lines: &mut Vec<String>, slow: bool, last: String)
 {
-    if last != input && !input.replace(' ', "").is_empty()
+    if last != input && !input.replace(' ', "").is_empty() && !input.starts_with('#')
     {
         if slow
         {
@@ -487,7 +494,8 @@ pub fn place_varxy(mut func: Vec<NumStr>, num: NumStr) -> Vec<NumStr>
                         sum.remove(0);
                     }
                     "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat" | "D"
-                    | "integrate" | "arclength" | "area" | "length" | "slope" | "lim" | "limit"
+                    | "integrate" | "arclength" | "∫" | "area" | "length" | "slope" | "lim"
+                    | "limit"
                         if i + 2 < func.len()
                             && if let Str(s) = &func[i + 2]
                             {
@@ -534,7 +542,8 @@ pub fn place_var(mut func: Vec<NumStr>, var: &str, num: NumStr) -> Vec<NumStr>
                         sum.remove(0);
                     }
                     "sum" | "summation" | "prod" | "product" | "Σ" | "Π" | "vec" | "mat" | "D"
-                    | "integrate" | "arclength" | "area" | "length" | "slope" | "lim" | "limit"
+                    | "integrate" | "arclength" | "∫" | "area" | "length" | "slope" | "lim"
+                    | "limit"
                         if i + 2 < func.len() && func[i + 2] == Str(var.to_string()) =>
                     {
                         i += 3;
@@ -597,9 +606,10 @@ pub fn parsed_to_string(
             {
                 let n = get_output(*options, colors, &n);
                 format!(
-                    "{}{}{}",
+                    "{}{}{}{}",
                     n.0,
                     n.1,
+                    n.2.unwrap_or_default(),
                     if options.color { "\x1b[0m" } else { "" }
                 )
             }
@@ -611,9 +621,10 @@ pub fn parsed_to_string(
                 {
                     num = get_output(*options, colors, &i);
                     str.push_str(&format!(
-                        "{}{}{},",
+                        "{}{}{}{},",
                         num.0,
                         num.1,
+                        num.2.unwrap_or_default(),
                         if options.color { "\x1b[0m" } else { "" }
                     ));
                 }
@@ -630,9 +641,10 @@ pub fn parsed_to_string(
                     {
                         num = get_output(*options, colors, &j);
                         str.push_str(&format!(
-                            "{}{}{},",
+                            "{}{}{}{},",
                             num.0,
                             num.1,
+                            num.2.unwrap_or_default(),
                             if options.color { "\x1b[0m" } else { "" }
                         ));
                     }
