@@ -1,13 +1,15 @@
 use crate::{
     complex::NumStr::{Matrix, Num, Vector},
-    help::help,
+    help::{help, help_for},
     load_vars::get_vars,
     math::do_math,
     misc::{parsed_to_string, to_output},
     parse::input_var,
     print::get_output,
     AngleType::{Degrees, Gradians, Radians},
-    Colors, Options, Variable,
+    Colors,
+    Notation::{LargeEngineering, Normal, Scientific, SmallEngineering},
+    Number, Options, Variable,
 };
 use crossterm::{
     execute, terminal,
@@ -23,7 +25,7 @@ pub fn arg_opts(
     options: &mut Options,
     colors: &mut Colors,
     args: &mut Vec<String>,
-) -> Result<(), &'static str>
+) -> Result<bool, &'static str>
 {
     args.remove(0);
     loop
@@ -46,9 +48,22 @@ pub fn arg_opts(
             }
             "-h" | "--help" =>
             {
-                help();
+                if args.len() > 1
+                {
+                    println!("{}", help_for(&args[1..].join(" ")));
+                }
+                else
+                {
+                    help();
+                }
                 process::exit(0)
             }
+            "--default" =>
+            {
+                args.remove(0);
+                *options = Options::default();
+                return Ok(true);
+            }
             "--" =>
             {
                 args.remove(0);
@@ -91,7 +106,7 @@ pub fn arg_opts(
             }
         }
     }
-    Ok(())
+    Ok(false)
 }
 pub fn file_opts(
     options: &mut Options,
@@ -149,6 +164,27 @@ pub fn set_commands(
 {
     match l
     {
+        "angle" =>
+        {
+            options.angle = match r
+            {
+                "deg" | "degree" | "degrees" => Degrees,
+                "rad" | "radians" | "radian" => Radians,
+                "grad" | "gradians" | "gradian" => Gradians,
+                _ => return Err("bad angle type"),
+            }
+        }
+        "notation" =>
+        {
+            options.notation = match r
+            {
+                "sci" | "scientific" | "s" | "10^" | "*10^" | "10" => Scientific,
+                "engSmall" | "e" => SmallEngineering,
+                "engLarge" | "eng" | "engineering" | "E" => LargeEngineering,
+                "normal" | "n" => Normal,
+                _ => return Err("bad notation type"),
+            }
+        }
         "re1col" => colors.re1col = r.to_string(),
         "im1col" => colors.im1col = r.to_string(),
         "re2col" => colors.re2col = r.to_string(),
@@ -165,6 +201,7 @@ pub fn set_commands(
         "promptc" => colors.prompt = "\x1b[".to_owned() + r,
         "imagc" => colors.imag = "\x1b[".to_owned() + r,
         "scic" => colors.sci = "\x1b[".to_owned() + r,
+        "unitsc" => colors.units = "\x1b[".to_owned() + r,
         "bracketc" =>
         {
             colors.brackets = r
@@ -204,12 +241,12 @@ pub fn set_commands(
                 return Err("Invalid point type");
             }
         }
-        "graphcli" | "slowcheck" | "tau" | "interactive" | "color" | "prompt" | "depth"
-        | "surface" | "flat" | "rt" | "small_e" | "sci" | "scientific" | "line" | "lines"
-        | "polar" | "frac" | "multi" | "tabbed" | "comma" | "graph" | "var_multiply"
-        | "scalegraph" | "debug" | "vars" | "onaxis" | "base" | "ticks" | "decimal" | "deci"
-        | "decimals" | "graphprec" | "graphprecision" | "prec" | "precision" | "range" | "xr"
-        | "yr" | "zr" | "vrange" | "vxr" | "vyr" | "vzr" | "frac_iter" | "2d" | "3d" =>
+        "graphcli" | "slowcheck" | "interactive" | "color" | "prompt" | "depth" | "surface"
+        | "flat" | "rt" | "line" | "lines" | "polar" | "frac" | "multi" | "tabbed" | "comma"
+        | "graph" | "units" | "scalegraph" | "debug" | "vars" | "onaxis" | "base" | "ticks"
+        | "decimal" | "deci" | "decimals" | "graphprec" | "graphprecision" | "prec"
+        | "windowsize" | "precision" | "range" | "xr" | "yr" | "zr" | "vrange" | "vxr" | "vyr"
+        | "vzr" | "2d" | "3d" =>
         {
             let mut args: Vec<f64> = Vec::new();
             {
@@ -237,6 +274,7 @@ pub fn set_commands(
                             args.push(
                                 do_math(parsed.0, *options, parsed.1)?
                                     .num()?
+                                    .number
                                     .real()
                                     .to_f64(),
                             );
@@ -260,6 +298,7 @@ pub fn set_commands(
                 args.push(
                     do_math(parsed.0, *options, parsed.1)?
                         .num()?
+                        .number
                         .real()
                         .to_f64(),
                 );
@@ -267,7 +306,6 @@ pub fn set_commands(
             match l
             {
                 "graphcli" => options.graph_cli = args[0] != 0.0,
-                "tau" => options.tau = args[0] != 0.0,
                 "color" => options.color = args[0] != 0.0,
                 "interactive" => options.stay_interactive = args[0] != 0.0,
                 "prompt" => options.prompt = args[0] != 0.0,
@@ -275,8 +313,6 @@ pub fn set_commands(
                 "surface" => options.surface = args[0] != 0.0,
                 "flat" => options.flat = args[0] != 0.0,
                 "rt" => options.real_time_output = args[0] != 0.0,
-                "small_e" => options.small_e = args[0] != 0.0,
-                "sci" | "scientific" => options.sci = args[0] != 0.0,
                 "line" | "lines" => options.lines = args[0] != 0.0,
                 "polar" => options.polar = args[0] != 0.0,
                 "frac" => options.frac = args[0] != 0.0,
@@ -284,21 +320,37 @@ pub fn set_commands(
                 "tabbed" => options.tabbed = args[0] != 0.0,
                 "comma" => options.comma = args[0] != 0.0,
                 "graph" => options.graph = args[0] != 0.0,
-                "var_multiply" => options.var_multiply = args[0] != 0.0,
+                "units" => options.units = args[0] != 0.0,
                 "scalegraph" => options.scale_graph = args[0] != 0.0,
                 "debug" => options.debug = args[0] != 0.0,
                 "vars" => options.allow_vars = args[0] != 0.0,
                 "onaxis" => options.onaxis = args[0] != 0.0,
                 "base" =>
                 {
-                    let n = args[0] as usize;
-                    if (2..=36).contains(&n)
+                    if args.len() == 2
                     {
-                        options.base = n
+                        let n1 = args[0] as i32;
+                        let n2 = args[1] as i32;
+                        if (2..=36).contains(&n1) && (2..=36).contains(&n2)
+                        {
+                            options.base = (n1, n2)
+                        }
+                        else
+                        {
+                            return Err("out of range of 2..=36");
+                        }
                     }
                     else
                     {
-                        return Err("out of range of 2..=36");
+                        let n = args[0] as i32;
+                        if (2..=36).contains(&n)
+                        {
+                            options.base = (n, n)
+                        }
+                        else
+                        {
+                            return Err("out of range of 2..=36");
+                        }
                     }
                 }
                 "ticks" => options.ticks = args[0],
@@ -315,14 +367,25 @@ pub fn set_commands(
                 }
                 "graphprec" | "graphprecision" => match args[0] as u32
                 {
-                    n if n != 0 => options.graph_prec = (n, n),
+                    n if n != 0 => options.graph_prec = n,
                     _ => return Err("Invalid graphprecision"),
                 },
+                "windowsize" =>
+                {
+                    if args.len() == 1
+                    {
+                        options.window_size = (args[0] as usize, args[0] as usize)
+                    }
+                    else
+                    {
+                        options.window_size = (args[0] as usize, args[1] as usize)
+                    }
+                }
                 "prec" | "precision" => match args[0] as u32
                 {
                     n if n != 0 =>
                     {
-                        options.prec = (n, n);
+                        options.prec = n;
                         if !vars.is_empty()
                         {
                             let v = get_vars(*options);
@@ -404,7 +467,10 @@ pub fn set_commands(
                                     else
                                     {
                                         vec![do_math(parsed.0, *options, parsed.1.clone())
-                                            .unwrap_or(Num(Complex::new(options.prec)))]
+                                            .unwrap_or(Num(Number::from(
+                                                Complex::new(options.prec),
+                                                None,
+                                            )))]
                                     };
                                     vars[i].funcvars = parsed.1;
                                     if var.name.contains(&'(')
@@ -629,7 +695,6 @@ pub fn set_commands(
                         options.vzr = (min, max)
                     }
                 }
-                "frac_iter" => options.frac_iter = args[0] as usize,
                 "2d" => options.samples_2d = args[0] as usize,
                 "3d" =>
                 {
@@ -657,7 +722,6 @@ pub fn silent_commands(options: &mut Options, input: &[char]) -> bool
     match input.iter().collect::<String>().as_str()
     {
         "default" | "defaults" | "reset" => *options = Options::default(),
-        "var_multiply" => options.var_multiply = !options.var_multiply,
         "interactive" => options.stay_interactive = !options.stay_interactive,
         "scalegraph" => options.scale_graph = !options.scale_graph,
         "debug" => options.debug = !options.debug,
@@ -667,14 +731,7 @@ pub fn silent_commands(options: &mut Options, input: &[char]) -> bool
         "onaxis" => options.onaxis = !options.onaxis,
         "surface" => options.surface = !options.surface,
         "flat" => options.flat = !options.flat,
-        "deg" => options.deg = Degrees,
-        "rad" => options.deg = Radians,
-        "grad" => options.deg = Gradians,
         "rt" => options.real_time_output = !options.real_time_output,
-        "tau" => options.tau = true,
-        "pi" => options.tau = false,
-        "small_e" => options.small_e = !options.small_e,
-        "sci" | "scientific" => options.sci = !options.sci,
         "line" | "lines" => options.lines = !options.lines,
         "polar" => options.polar = !options.polar,
         "frac" => options.frac = !options.frac,
@@ -682,6 +739,7 @@ pub fn silent_commands(options: &mut Options, input: &[char]) -> bool
         "tabbed" => options.tabbed = !options.tabbed,
         "comma" => options.comma = !options.comma,
         "graph" => options.graph = !options.graph,
+        "units" => options.graph = !options.units,
         "vars" => options.allow_vars = !options.allow_vars,
         "graphcli" => options.graph_cli = !options.graph_cli,
         _ => return false,
@@ -698,7 +756,7 @@ pub fn commands(
     stdout: &mut Stdout,
 )
 {
-    match input.iter().collect::<String>().as_str()
+    match input.iter().collect::<String>().trim_start().trim_end()
     {
         "graphcli" =>
         {
@@ -712,12 +770,6 @@ pub fn commands(
             stdout.flush().unwrap();
             *options = Options::default();
         }
-        "var_multiply" =>
-        {
-            print!("\x1b[G\x1b[A\x1b[K");
-            stdout.flush().unwrap();
-            options.var_multiply = !options.var_multiply
-        }
         "scalegraph" =>
         {
             print!("\x1b[G\x1b[A\x1b[K");
@@ -760,44 +812,12 @@ pub fn commands(
             stdout.flush().unwrap();
             options.flat = !options.flat;
         }
-        "deg" =>
-        {
-            print!("\x1b[G\x1b[A\x1b[K");
-            stdout.flush().unwrap();
-            options.deg = Degrees;
-        }
-        "rad" =>
-        {
-            print!("\x1b[G\x1b[A\x1b[K");
-            stdout.flush().unwrap();
-            options.deg = Radians;
-        }
-        "grad" =>
-        {
-            print!("\x1b[G\x1b[A\x1b[K");
-            stdout.flush().unwrap();
-            options.deg = Gradians;
-        }
         "rt" =>
         {
             print!("\x1b[G\x1b[A\x1b[K");
             stdout.flush().unwrap();
             options.real_time_output = !options.real_time_output;
         }
-        "tau" => options.tau = true,
-        "pi" => options.tau = false,
-        "small_e" =>
-        {
-            print!("\x1b[G\x1b[A\x1b[K");
-            stdout.flush().unwrap();
-            options.small_e = !options.small_e;
-        }
-        "sci" | "scientific" =>
-        {
-            print!("\x1b[G\x1b[A\x1b[K");
-            stdout.flush().unwrap();
-            options.sci = !options.sci;
-        }
         "clear" =>
         {
             execute!(stdout, Clear(ClearType::Purge)).unwrap();
@@ -858,6 +878,12 @@ pub fn commands(
             stdout.flush().unwrap();
             options.graph = !options.graph;
         }
+        "units" =>
+        {
+            print!("\x1b[G\x1b[A\x1b[K");
+            stdout.flush().unwrap();
+            options.units = !options.units;
+        }
         "history" =>
         {
             print!("\x1b[G\x1b[A\x1b[K");
@@ -892,10 +918,11 @@ pub fn commands(
                         {
                             let n = get_output(*options, colors, n);
                             print!(
-                                "{}={}{}{}\x1b[G\n",
+                                "{}={}{}{}{}\x1b[G\n",
                                 v.name.iter().collect::<String>(),
                                 n.0,
                                 n.1,
+                                n.2.unwrap_or_default(),
                                 if options.color { &colors.text } else { "" }
                             )
                         }
@@ -907,6 +934,7 @@ pub fn commands(
                                 let n = get_output(*options, colors, i);
                                 st.push_str(&n.0);
                                 st.push_str(&n.1);
+                                st.push_str(&n.2.unwrap_or_default());
                                 if options.color
                                 {
                                     st.push_str(&colors.text)
@@ -930,6 +958,7 @@ pub fn commands(
                                     let n = get_output(*options, colors, g);
                                     st.push_str(&n.0);
                                     st.push_str(&n.1);
+                                    st.push_str(&n.2.unwrap_or_default());
                                     if options.color
                                     {
                                         st.push_str(&colors.text)
@@ -965,7 +994,28 @@ pub fn commands(
             process::exit(0);
         }
         _ =>
-        {}
+        {
+            let n = input.iter().collect::<String>();
+            let mut split = n.splitn(2, ' ');
+            match split.next().unwrap()
+            {
+                "history" | "his" =>
+                {
+                    print!("\x1b[A\x1b[G\x1b[K");
+                    let r = split.next().unwrap();
+                    for i in lines
+                    {
+                        if i.contains(r)
+                        {
+                            print!("{}\x1b[G\n", i);
+                        }
+                    }
+                    stdout.flush().unwrap();
+                }
+                _ =>
+                {}
+            }
+        }
     }
 }
 pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, last: &str)
@@ -973,8 +1023,10 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
 {
     match l.replace(' ', "").as_str()
     {
-        "colors" => format!(
-            "{}textc={} {}promptc={} {}imagc={} {}scic={} \x1b[0mbracketc={} \x1b[0mre1col={} re2col={} re3col={} re4col={} re5col={} re6col={} im1col={} im2col={} im3col={} im4col={} im5col={} im6col={}",
+        "colors" =>
+        {
+            format!(
+            "{}textc={} {}promptc={} {}imagc={} {}scic={} {}unitsc={} \x1b[0mbracketc={} \x1b[0mre1col={} re2col={} re3col={} re4col={} re5col={} re6col={} im1col={} im2col={} im3col={} im4col={} im5col={} im6col={}",
             colors.text,
             &colors.text[2..],
             colors.prompt,
@@ -983,7 +1035,9 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
             &colors.imag[2..],
             colors.sci,
             &colors.sci[2..],
-            colors.brackets.iter().fold(String::new(), |out, a| out + a+ &a[2..]+",").trim_end_matches(','),
+            colors.units,
+            &colors.units[2..],
+            bracketcol(&colors.brackets),
             formatcol(&colors.re1col),
             formatcol(&colors.re2col),
             formatcol(&colors.re3col),
@@ -996,8 +1050,8 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
             formatcol(&colors.im4col),
             formatcol(&colors.im5col),
             formatcol(&colors.im6col),
-        ),
-        "var_multiply" => format!("{}", options.var_multiply),
+        )
+        }
         "slowcheck" => format!("{}", options.slowcheck),
         "label" => format!("{},{},{}", colors.label.0, colors.label.1, colors.label.2),
         "color" => format!("{}", options.color),
@@ -1006,7 +1060,6 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "flat" => format!("{}", options.flat),
         "prompt" => format!("{}", options.prompt),
         "rt" => format!("{}", options.real_time_output),
-        "sci" | "scientific" => format!("{}", options.sci),
         "debug" => format!("{}", options.debug),
         "scalegraph" => format!("{}", options.scale_graph),
         "line" => format!("{}", options.lines),
@@ -1016,14 +1069,16 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
         "tabbed" => format!("{}", options.tabbed),
         "comma" => format!("{}", options.comma),
         "graph" => format!("{}", options.graph),
+        "units" => format!("{}", options.units),
         "graphcli" => format!("{}", options.graph_cli),
         "point" => format!("{}", options.point_style),
-        "base" => format!("{}", options.base),
+        "base" => format!("{} {}", options.base.0, options.base.1),
         "ticks" => format!("{}", options.ticks),
         "onaxis" => format!("{}", options.onaxis),
         "decimal" | "deci" | "decimals" => format!("{}", options.decimal_places),
-        "prec" | "precision" => format!("{}", options.prec.0),
-        "graphprec" | "graphprecision" => format!("{}", options.graph_prec.0),
+        "prec" | "precision" => format!("{}", options.prec),
+        "windowsize" => format!("{},{}", options.window_size.0, options.window_size.1),
+        "graphprec" | "graphprecision" => format!("{}", options.graph_prec),
         "xr" => format!("{},{}", options.xr.0, options.xr.1),
         "yr" => format!("{},{}", options.yr.0, options.yr.1),
         "zr" => format!("{},{}", options.zr.0, options.zr.1),
@@ -1043,23 +1098,30 @@ pub fn equal_to(options: Options, colors: &Colors, vars: &[Variable], l: &str, l
             options.vzr.0,
             options.vzr.1
         ),
-        "frac_iter" => format!("{}", options.frac_iter),
         "2d" => format!("{}", options.samples_2d),
         "3d" => format!("{} {}", options.samples_3d.0, options.samples_3d.1),
-        "deg" => format!("{}", options.deg == Degrees),
-        "grad" => format!("{}", options.deg == Gradians),
-        "rad" => format!("{}", options.deg == Radians),
+        "angle" => match options.angle
+        {
+            Degrees => "deg",
+            Radians => "rad",
+            Gradians => "grad",
+        }
+        .to_string(),
+        "notation" => match options.notation
+        {
+            SmallEngineering => "e",
+            LargeEngineering => "E",
+            Scientific => "s",
+            Normal => "n",
+        }
+        .to_string(),
         "interactive" => format!("{}", options.interactive),
         "textc" => colors.text.to_string(),
         "promptc" => colors.prompt.to_string(),
         "imagc" => colors.imag.to_string(),
         "scic" => colors.sci.to_string(),
-        "bracketc" => colors
-            .brackets
-            .iter()
-            .fold(String::new(), |out, a| out + a+ &a[2..]+",")
-            .trim_end_matches(',')
-            .to_string(),
+        "unitsc" => colors.units.to_string(),
+        "bracketc" => bracketcol(&colors.brackets),
         "re1col" => formatcol(&colors.re1col),
         "re2col" => formatcol(&colors.re2col),
         "re3col" => formatcol(&colors.re3col),
@@ -1106,3 +1168,11 @@ fn formatcol(color: &str) -> String
         color
     )
 }
+fn bracketcol(bracket: &[String]) -> String
+{
+    bracket
+        .iter()
+        .fold(String::new(), |out, a| out + a + &a[2..] + ",")
+        .trim_end_matches(',')
+        .to_string()
+}
