@@ -4,6 +4,7 @@ use crate::{
         NumStr::{Matrix, Num, Vector},
     },
     fraction::fraction,
+    help::help_for,
     load_vars::set_commands_or_vars,
     math::do_math,
     misc::{
@@ -13,10 +14,12 @@ use crate::{
     options::{equal_to, silent_commands},
     parse::input_var,
     AngleType::{Degrees, Gradians, Radians},
-    Colors, Options, Variable,
+    Colors,
+    Notation::{LargeEngineering, Normal, Scientific, SmallEngineering},
+    Number, Options, Variable,
 };
 use rug::{float::Constant::Pi, ops::CompleteRound, Complex, Float, Integer};
-use std::{cmp::Ordering, str::FromStr};
+use std::cmp::Ordering;
 #[allow(clippy::too_many_arguments)]
 pub fn print_concurrent(
     unmodified_input: &[char],
@@ -65,7 +68,38 @@ pub fn print_concurrent(
             }
         }
         let tempinput = unparsed.iter().collect::<String>();
-        if tempinput.ends_with('=')
+        if tempinput.starts_with("help ")
+        {
+            let message = help_for(tempinput.splitn(2, ' ').last().unwrap());
+            let mut num = message.chars().filter(|c| c == &'\n').count();
+            {
+                let width = get_terminal_dimensions().0;
+                let mut len = 0;
+                for i in no_col(&message, options.color)
+                {
+                    len += 1;
+                    if i == '\n'
+                    {
+                        len = 0
+                    }
+                    else if len > width
+                    {
+                        len = 0;
+                        num += 1;
+                    }
+                }
+            }
+            print!(
+                "\x1b[G\x1b[J\n{}\x1b[G\x1b[{}A{}{}{}",
+                message,
+                num + 1,
+                prompt(options, &colors),
+                to_output(&unmodified_input[start..end], options.color, &colors),
+                if options.color { "\x1b[0m" } else { "" },
+            );
+            return (num, false, false, false);
+        }
+        else if tempinput.ends_with('=')
         {
             let out = equal_to(
                 options,
@@ -155,95 +189,132 @@ pub fn print_concurrent(
         {
             let n = unparsed.iter().position(|c| c == &'=').unwrap_or(0) + 1;
             let mut input = unparsed[n..].iter().collect::<String>();
-            let mut func = unparsed[..n].to_vec();
-            let mut func_vars: Vec<(isize, String)> = Vec::new();
-            if func.contains(&'(')
-            {
-                func.drain(0..=func.iter().position(|c| c == &'(').unwrap());
-                func.pop();
-                func.pop();
-                for i in func.split(|c| c == &',')
-                {
-                    func_vars.push((-1, i.iter().collect()));
-                }
-            }
-            if input.contains(':')
-            {
-                let inp = input;
-                let mut split = inp.split(':').collect::<Vec<&str>>();
-                input = split.pop().unwrap().to_string();
-                for i in split
-                {
-                    if i.contains('=')
-                    {
-                        let mut split = i.splitn(2, '=');
-                        func_vars.push((-1, split.next().unwrap().to_string()));
-                    }
-                }
-            }
-            let out = match input_var(
-                &input,
-                vars.to_vec(),
-                &mut func_vars,
-                &mut 0,
-                options,
-                false,
-                true,
-                0,
-                Vec::new(),
+            let mut func = unparsed[..n - 1].to_vec();
+            return if matches!(
+                func.iter().collect::<String>().as_str(),
+                "angle"
+                    | "re1col"
+                    | "im1col"
+                    | "re2col"
+                    | "im2col"
+                    | "re3col"
+                    | "im3col"
+                    | "re4col"
+                    | "im4col"
+                    | "re5col"
+                    | "im5col"
+                    | "re6col"
+                    | "im6col"
+                    | "notation"
+                    | "textc"
+                    | "promptc"
+                    | "imagc"
+                    | "scic"
+                    | "unitsc"
+                    | "bracketc"
+                    | "label"
             )
-            {
-                Ok(n) => parsed_to_string(n.0, n.1, &options, &colors),
-                _ => String::new(),
-            };
-            if out.is_empty()
-            {
-                clear(unmodified_input, start, end, options, &colors);
-                return (0, false, false, true);
-            }
-            let (width, height) = get_terminal_dimensions();
-            let len = no_col(&out, options.color).len();
-            let wrap = (len - 1) / width + 1;
-            return if len > width * (height - 1)
-            {
-                if long_output
-                {
-                    print!(
-                        "\x1b[G\n\x1b[J{}{}",
-                        out,
-                        if options.color { "\x1b[0m" } else { "" }
-                    );
-                    (wrap, true, false, true)
-                }
-                else
-                {
-                    print!(
-                        "\x1b[J\x1b[G\ntoo long, will print on enter\x1b[G\x1b[A\x1b[K{}{}{}",
-                        prompt(options, &colors),
-                        to_output(&unmodified_input[start..end], options.color, &colors),
-                        if options.color { "\x1b[0m" } else { "" },
-                    );
-                    (0, true, true, true)
-                }
-            }
-            else
             {
                 print!(
-                    "\x1b[G\n\x1b[J{}{}\x1b[G\x1b[K{}{}{}",
-                    out,
-                    if wrap == 0
-                    {
-                        String::new()
-                    }
-                    else
-                    {
-                        format!("\x1b[{}A", wrap)
-                    },
+                    "\x1b[G\n\x1b[J{}\x1b[A\x1b[G\x1b[K{}{}{}",
+                    input,
                     prompt(options, &colors),
                     to_output(&unmodified_input[start..end], options.color, &colors),
                     if options.color { "\x1b[0m" } else { "" }
                 );
-                (wrap, true, false, true)
+                (1, true, false, true)
+            }
+            else
+            {
+                let mut func_vars: Vec<(isize, String)> = Vec::new();
+                if func.contains(&'(')
+                {
+                    func.drain(0..=func.iter().position(|c| c == &'(').unwrap());
+                    func.pop();
+                    func.pop();
+                    for i in func.split(|c| c == &',')
+                    {
+                        func_vars.push((-1, i.iter().collect()));
+                    }
+                }
+                if input.contains(':')
+                {
+                    let inp = input;
+                    let mut split = inp.split(':').collect::<Vec<&str>>();
+                    input = split.pop().unwrap().to_string();
+                    for i in split
+                    {
+                        if i.contains('=')
+                        {
+                            let mut split = i.splitn(2, '=');
+                            func_vars.push((-1, split.next().unwrap().to_string()));
+                        }
+                    }
+                }
+                let out = match input_var(
+                    &input,
+                    vars.to_vec(),
+                    &mut func_vars,
+                    &mut 0,
+                    options,
+                    false,
+                    true,
+                    0,
+                    Vec::new(),
+                )
+                {
+                    Ok(n) => parsed_to_string(n.0, n.1, &options, &colors),
+                    _ => String::new(),
+                };
+                if out.is_empty()
+                {
+                    clear(unmodified_input, start, end, options, &colors);
+                    return (0, false, false, true);
+                }
+                let (width, height) = get_terminal_dimensions();
+                let len = no_col(&out, options.color).len();
+                let wrap = (len - 1) / width + 1;
+                if len > width * (height - 1)
+                {
+                    if long_output
+                    {
+                        print!(
+                            "\x1b[G\n\x1b[J{}{}",
+                            out,
+                            if options.color { "\x1b[0m" } else { "" }
+                        );
+                        (wrap, true, false, true)
+                    }
+                    else
+                    {
+                        print!(
+                            "\x1b[J\x1b[G\ntoo long, will print on enter\x1b[G\x1b[A\x1b[K{}{}{}",
+                            prompt(options, &colors),
+                            to_output(&unmodified_input[start..end], options.color, &colors),
+                            if options.color { "\x1b[0m" } else { "" },
+                        );
+                        (0, true, true, true)
+                    }
+                }
+                else
+                {
+                    print!(
+                        "\x1b[G\n\x1b[J{}{}\x1b[G\x1b[K{}{}{}",
+                        out,
+                        if wrap == 0
+                        {
+                            String::new()
+                        }
+                        else
+                        {
+                            format!("\x1b[{}A", wrap)
+                        },
+                        prompt(options, &colors),
+                        to_output(&unmodified_input[start..end], options.color, &colors),
+                        if options.color { "\x1b[0m" } else { "" }
+                    );
+                    (wrap, true, false, true)
+                }
             };
         }
     }
@@ -461,9 +532,10 @@ pub fn print_concurrent(
     {
         Num(n) =>
         {
-            let output = get_output(options, &colors, &n);
-            let (frac_a, frac_b) = if options.frac || options.frac_iter == 0
+            let mut output = get_output(options, &colors, &n);
+            let (mut frac_a, frac_b) = if options.frac
             {
+                let n = n.number;
                 let fa = fraction(n.real().clone(), options);
                 let fb = fraction(n.imag().clone(), options);
                 let sign = if !output.0.is_empty() && n.imag().is_sign_positive()
@@ -567,6 +639,11 @@ pub fn print_concurrent(
                 (String::new(), String::new())
             };
             let (width, height) = get_terminal_dimensions();
+            if let Some(st) = output.2
+            {
+                output.0 += &st;
+                frac_a += &st;
+            }
             let len1 = no_col(&output.0, options.color).len();
             let len2 = no_col(&output.1, options.color).len();
             if (frac == 1 && !options.frac)
@@ -700,7 +777,7 @@ pub fn print_concurrent(
             {
                 v = to_polar(
                     v,
-                    match options.deg
+                    match options.angle
                     {
                         Degrees => 180 / Complex::with_val(options.prec, Pi),
                         Radians => Complex::with_val(options.prec, 1),
@@ -715,7 +792,8 @@ pub fn print_concurrent(
             for (k, i) in v.iter().enumerate()
             {
                 out = get_output(options, &colors, i);
-                if options.frac || options.frac_iter == 0
+                let i = &i.number;
+                if options.frac
                 {
                     frac_temp = fraction(i.real().clone(), options);
                     frac_out += if !frac_temp.is_empty()
@@ -766,6 +844,11 @@ pub fn print_concurrent(
                 }
                 output += &out.0;
                 output += &out.1;
+                if let Some(st) = out.2
+                {
+                    output += &st;
+                    frac_out += &st;
+                }
                 if options.color
                 {
                     output += "\x1b[0m";
@@ -874,7 +957,8 @@ pub fn print_concurrent(
                 for (k, i) in j.iter().enumerate()
                 {
                     out = get_output(options, &colors, i);
-                    if options.frac || options.frac_iter == 0
+                    let i = &i.number;
+                    if options.frac
                     {
                         frac_temp = fraction(i.real().clone(), options);
                         frac_out += if !frac_temp.is_empty()
@@ -925,6 +1009,11 @@ pub fn print_concurrent(
                     }
                     output += &out.0;
                     output += &out.1;
+                    if let Some(st) = out.2
+                    {
+                        output += &st;
+                        frac_out += &st;
+                    }
                     if options.color
                     {
                         output += "\x1b[0m";
@@ -1015,7 +1104,7 @@ pub fn print_concurrent(
                         num += 1;
                     }
                 }
-                frac_out += "\x1b[K\n";
+                frac_out += "\x1b[K\x1b[G\n\x1b[K";
             }
             if length > width * (height - 1) || num > (height - 2)
             {
@@ -1103,8 +1192,9 @@ pub fn print_answer(num: NumStr, options: Options, colors: &Colors)
         {
             let a = get_output(options, colors, &n);
             print!(
-                "{}{}{}",
+                "{}{}{}{}",
                 a.0,
+                a.2.unwrap_or_default(),
                 a.1,
                 if options.color { "\x1b[0m" } else { "" }
             );
@@ -1115,7 +1205,7 @@ pub fn print_answer(num: NumStr, options: Options, colors: &Colors)
             {
                 v = to_polar(
                     v,
-                    match options.deg
+                    match options.angle
                     {
                         Degrees => 180 / Complex::with_val(options.prec, Pi),
                         Radians => Complex::with_val(options.prec, 1),
@@ -1130,6 +1220,7 @@ pub fn print_answer(num: NumStr, options: Options, colors: &Colors)
                 out = get_output(options, colors, i);
                 output += out.0.as_str();
                 output += out.1.as_str();
+                output += &out.2.unwrap_or_default();
                 if options.color
                 {
                     output += "\x1b[0m";
@@ -1174,6 +1265,7 @@ pub fn print_answer(num: NumStr, options: Options, colors: &Colors)
                     out = get_output(options, colors, i);
                     output += out.0.as_str();
                     output += out.1.as_str();
+                    output += &out.2.unwrap_or_default();
                     if options.color
                     {
                         output += "\x1b[0m";
@@ -1216,8 +1308,14 @@ pub fn print_answer(num: NumStr, options: Options, colors: &Colors)
         {}
     }
 }
-pub fn get_output(options: Options, colors: &Colors, num: &Complex) -> (String, String)
+pub fn get_output(
+    options: Options,
+    colors: &Colors,
+    number: &Number,
+) -> (String, String, Option<String>)
 {
+    let num = number.number.clone();
+    let units = number.units;
     let dec = if options.decimal_places == 0
     {
         1
@@ -1226,210 +1324,247 @@ pub fn get_output(options: Options, colors: &Colors, num: &Complex) -> (String,
     {
         options.decimal_places
     };
-    if options.base != 10
+    if options.notation != Normal
     {
-        let sign = if num.imag().is_sign_positive() && !num.real().is_zero()
+        if options.base.1 != 10
         {
-            "+"
-        }
-        else
-        {
-            ""
-        }
-        .to_string();
-        (
-            if !num.real().is_zero()
+            let sign = if num.imag().is_sign_positive() && !num.real().is_zero()
             {
-                let n = num.real().to_string_radix(options.base as i32, None);
-                if n.contains('e')
-                {
-                    n
-                }
-                else
-                {
-                    n.trim_end_matches('0').trim_end_matches('.').to_string()
-                }
-            }
-            else if num.imag().is_zero()
-            {
-                "0".to_string()
+                "+"
             }
             else
             {
-                String::new()
-            },
-            if !num.imag().is_zero()
-            {
-                let n = num.imag().to_string_radix(options.base as i32, None);
-                sign + &if n.contains('e')
-                {
-                    n
-                }
-                else
-                {
-                    n.trim_end_matches('0').trim_end_matches('.').to_string()
-                } + &if options.color
-                {
-                    format!("{}i", &colors.imag)
-                }
-                else
-                {
-                    "i".to_string()
-                }
+                ""
             }
-            else
-            {
-                String::new()
-            },
-        )
-    }
-    else if options.sci
-    {
-        let sign = if num.imag().is_sign_positive() && !num.real().is_zero()
-        {
-            "+"
-        }
-        else
-        {
-            ""
-        }
-        .to_string();
-        (
-            if num.real().is_zero() && !num.imag().is_zero()
-            {
-                String::new()
-            }
-            else
-            {
-                if options.comma
+            .to_string();
+            (
+                if !num.real().is_zero()
                 {
-                    add_commas(&remove_trailing_zeros(
-                        &format!("{:e}", num.real()),
-                        dec,
-                        options.prec,
-                    ))
-                }
-                else
-                {
-                    remove_trailing_zeros(&format!("{:e}", num.real()), dec, options.prec)
-                }
-                .replace("e0", "")
-                .replace(
-                    'e',
-                    &if options.small_e
+                    let n = num
+                        .real()
+                        .to_string_radix(options.base.1, Some(options.decimal_places));
+                    if n.chars().filter(|c| *c == '@').count() == 1
                     {
-                        if options.color
-                        {
-                            format!("{}e", &colors.sci)
-                        }
-                        else
-                        {
-                            "e".to_string()
-                        }
+                        n.replace('@', &notate(options, colors))
+                            + if options.color { "\x1b[0m" } else { "" }
                     }
-                    else if options.color
+                    else if n.contains('e')
                     {
-                        format!("{}E", &colors.sci)
+                        n.replace("e0", "").replace('e', &notate(options, colors))
+                            + if options.color { "\x1b[0m" } else { "" }
                     }
                     else
                     {
-                        "E".to_string()
-                    },
-                ) + if options.color { "\x1b[0m" } else { "" }
-            },
-            if num.imag().is_zero()
-            {
-                String::new()
-            }
-            else if num.imag() == &1
-            {
-                sign + &if options.color
-                {
-                    format!("{}i", &colors.imag)
-                }
-                else
-                {
-                    "i".to_string()
-                }
-            }
-            else if num.imag() == &-1
-            {
-                sign + &if options.color
-                {
-                    format!("-{}i", &colors.imag)
-                }
-                else
-                {
-                    "i".to_string()
-                }
-            }
-            else
-            {
-                if options.comma
-                {
-                    add_commas(
-                        &(sign
-                            + &remove_trailing_zeros(
-                                &format!("{:e}", num.imag(),),
-                                dec,
-                                options.prec,
-                            )),
-                    )
-                }
-                else
-                {
-                    sign + &remove_trailing_zeros(&format!("{:e}", num.imag(),), dec, options.prec)
-                }
-                .replace("e0", "")
-                .replace(
-                    'e',
-                    &if options.small_e
-                    {
-                        if options.color
-                        {
-                            format!("{}e", &colors.sci)
-                        }
-                        else
-                        {
-                            "e".to_string()
-                        }
+                        n.trim_end_matches('0').trim_end_matches('.').to_string()
                     }
-                    else if options.color
+                }
+                else if num.imag().is_zero()
+                {
+                    "0".to_string()
+                }
+                else
+                {
+                    String::new()
+                },
+                if !num.imag().is_zero()
+                {
+                    let n = num
+                        .imag()
+                        .to_string_radix(options.base.1, Some(options.decimal_places));
+                    sign + &if n.chars().filter(|c| *c == '@').count() == 1
                     {
-                        format!("{}E", &colors.sci)
+                        n.replace('@', &notate(options, colors))
+                            + if options.color { "\x1b[0m" } else { "" }
+                    }
+                    else if n.contains('e')
+                    {
+                        n.replace("e0", "").replace('e', &notate(options, colors))
+                            + if options.color { "\x1b[0m" } else { "" }
                     }
                     else
                     {
-                        "E".to_string()
-                    },
-                ) + &if options.color
-                {
-                    format!("{}i", &colors.imag)
+                        n.trim_end_matches('0').trim_end_matches('.').to_string()
+                    } + &if options.color
+                    {
+                        format!("{}i", &colors.imag)
+                    }
+                    else
+                    {
+                        "i".to_string()
+                    }
                 }
                 else
                 {
-                    "i".to_string()
+                    String::new()
+                },
+                if options.units && num.imag().is_zero()
+                {
+                    units.map(|units| {
+                        format!(
+                            "{}{}{}",
+                            if options.color
+                            {
+                                "\x1b[".to_owned() + &colors.units
+                            }
+                            else
+                            {
+                                String::new()
+                            },
+                            units.to_string(options),
+                            if options.color { "\x1b[0m" } else { "" }
+                        )
+                    })
                 }
-            },
-        )
+                else
+                {
+                    None
+                },
+            )
+        }
+        else
+        {
+            let sign = if num.imag().is_sign_positive() && !num.real().is_zero()
+            {
+                "+"
+            }
+            else
+            {
+                ""
+            }
+            .to_string();
+            (
+                if num.real().is_zero() && !num.imag().is_zero()
+                {
+                    String::new()
+                }
+                else
+                {
+                    if options.comma
+                    {
+                        add_commas(&remove_trailing_zeros(
+                            &format!("{:e}", num.real()),
+                            dec,
+                            options.prec,
+                        ))
+                    }
+                    else
+                    {
+                        remove_trailing_zeros(&format!("{:e}", num.real()), dec, options.prec)
+                    }
+                    .replace("e0", "")
+                    .replace('e', &notate(options, colors))
+                        + if options.color { "\x1b[0m" } else { "" }
+                },
+                if num.imag().is_zero()
+                {
+                    String::new()
+                }
+                else if num.imag() == &1
+                {
+                    sign + &if options.color
+                    {
+                        format!("{}i", &colors.imag)
+                    }
+                    else
+                    {
+                        "i".to_string()
+                    }
+                }
+                else if num.imag() == &-1
+                {
+                    sign + &if options.color
+                    {
+                        format!("-{}i", &colors.imag)
+                    }
+                    else
+                    {
+                        "i".to_string()
+                    }
+                }
+                else
+                {
+                    if options.comma
+                    {
+                        add_commas(
+                            &(sign
+                                + &remove_trailing_zeros(
+                                    &format!("{:e}", num.imag()),
+                                    dec,
+                                    options.prec,
+                                )),
+                        )
+                    }
+                    else
+                    {
+                        sign + &remove_trailing_zeros(
+                            &format!("{:e}", num.imag()),
+                            dec,
+                            options.prec,
+                        )
+                    }
+                    .replace("e0", "")
+                    .replace('e', &notate(options, colors))
+                        + &if options.color
+                        {
+                            format!("{}i", &colors.imag)
+                        }
+                        else
+                        {
+                            "i".to_string()
+                        }
+                },
+                if options.units && num.imag().is_zero()
+                {
+                    units.map(|units| {
+                        format!(
+                            "{}{}{}",
+                            if options.color
+                            {
+                                "\x1b[".to_owned() + &colors.units
+                            }
+                            else
+                            {
+                                String::new()
+                            },
+                            units.to_string(options),
+                            if options.color { "\x1b[0m" } else { "" }
+                        )
+                    })
+                }
+                else
+                {
+                    None
+                },
+            )
+        }
     }
     else
     {
         let re = if options.comma
         {
-            add_commas(&to_string(num.real(), options.decimal_places, false))
+            add_commas(&to_string(
+                num.real(),
+                options.decimal_places,
+                false,
+                options.base.1,
+            ))
         }
         else
         {
-            to_string(num.real(), options.decimal_places, false)
+            to_string(num.real(), options.decimal_places, false, options.base.1)
         };
         let im = if options.comma
         {
-            add_commas(&to_string(num.imag(), options.decimal_places, true))
+            add_commas(&to_string(
+                num.imag(),
+                options.decimal_places,
+                true,
+                options.base.1,
+            ))
         }
         else
         {
-            to_string(num.imag(), options.decimal_places, true)
+            to_string(num.imag(), options.decimal_places, true, options.base.1)
         };
         let sign = if num.imag().is_sign_positive() && re != "0"
         {
@@ -1487,12 +1622,34 @@ pub fn get_output(options: Options, colors: &Colors, num: &Complex) -> (String,
                         "i".to_string()
                     }
             },
+            if options.units && num.imag().is_zero()
+            {
+                units.map(|units| {
+                    format!(
+                        "{}{}{}",
+                        if options.color
+                        {
+                            "\x1b[".to_owned() + &colors.units
+                        }
+                        else
+                        {
+                            String::new()
+                        },
+                        units.to_string(options),
+                        if options.color { "\x1b[0m" } else { "" }
+                    )
+                })
+            }
+            else
+            {
+                None
+            },
         )
     }
 }
-fn to_string(num: &Float, decimals: usize, imag: bool) -> String
+fn to_string(num: &Float, decimals: usize, imag: bool, radix: i32) -> String
 {
-    let (neg, mut str, exp) = num.to_sign_string_exp(10, None);
+    let (neg, mut str, exp) = num.to_sign_string_exp(radix, None);
     let mut neg = if neg { "-" } else { "" };
     if exp.is_none()
     {
@@ -1527,7 +1684,7 @@ fn to_string(num: &Float, decimals: usize, imag: bool) -> String
     {
         return if str == "0"
         {
-            "0".to_string()
+            neg.to_owned() + "0"
         }
         else
         {
@@ -1539,6 +1696,10 @@ fn to_string(num: &Float, decimals: usize, imag: bool) -> String
         str.push_str(&"0".repeat(exp as usize - str.len()));
     }
     let mut zeros = String::new();
+    if -exp as i128 > decimals as i128
+    {
+        return neg.to_owned() + "0";
+    }
     if exp < 0
     {
         zeros = "0".repeat(-exp as usize);
@@ -1556,7 +1717,7 @@ fn to_string(num: &Float, decimals: usize, imag: bool) -> String
     {
         return if str == "0"
         {
-            "0".to_string()
+            neg.to_owned() + "0"
         }
         else
         {
@@ -1574,28 +1735,31 @@ fn to_string(num: &Float, decimals: usize, imag: bool) -> String
             r.insert(decimals, '.');
         }
     }
-    let mut d = Float::with_val(num.prec(), Float::parse(&r).unwrap())
+    let mut d = Float::parse_radix(&r, radix)
+        .unwrap()
+        .complete(num.prec())
         .to_integer()
         .unwrap();
     if exp > 0
     {
         zeros = "0".repeat(r.to_string().len() - r.to_string().trim_start_matches('0').len());
-        if d.to_string() == 10.0f64.powi(decimals as i32 - 1).to_string()
+        if d.to_string() == (radix as f64).powi(decimals as i32 - 1).to_string()
         {
             zeros.pop();
         }
     }
-    if d.to_string().trim_end_matches('0') == "1"
+    if radix == 10
+        && d.to_string().trim_end_matches('0') == "1"
         && r.trim_start_matches('0')
             .trim_start_matches('.')
             .starts_with('9')
     {
         if zeros.is_empty()
         {
-            let t: Float = Float::with_val(
-                num.prec(),
-                Float::parse(if l.is_empty() { "0" } else { &l }).unwrap(),
-            ) + 1;
+            let t: Float = Float::parse_radix(if l.is_empty() { "0" } else { &l }, radix)
+                .unwrap()
+                .complete(num.prec())
+                + 1;
             l = t.to_integer().unwrap().to_string();
             d = Integer::new();
         }
@@ -1610,12 +1774,19 @@ fn to_string(num: &Float, decimals: usize, imag: bool) -> String
     }
     if decimals == 0
     {
-        if zeros.is_empty() && d.to_string().chars().next().unwrap().to_digit(10).unwrap() == 1
+        if zeros.is_empty()
+            && d.to_string_radix(radix)
+                .chars()
+                .next()
+                .unwrap()
+                .to_digit(radix as u32)
+                .unwrap()
+                == 1
         {
             format!(
                 "{}{}",
                 neg,
-                Integer::from_str(&l).unwrap_or(Integer::new()) + 1
+                Integer::from_str_radix(&l, radix).unwrap_or_default() + 1
             )
         }
         else
@@ -1630,7 +1801,7 @@ fn to_string(num: &Float, decimals: usize, imag: bool) -> String
             neg,
             if l.is_empty() { "0" } else { &l },
             zeros,
-            d
+            d.to_string_radix(radix)
         )
         .trim_end_matches('0')
         .trim_end_matches('.')
@@ -1688,7 +1859,7 @@ fn add_commas(input: &str) -> String
     }
     result.chars().rev().collect::<String>()
 }
-fn remove_trailing_zeros(input: &str, dec: usize, prec: (u32, u32)) -> String
+fn remove_trailing_zeros(input: &str, dec: usize, prec: u32) -> String
 {
     let pos = match input.find('e')
     {
@@ -1699,7 +1870,18 @@ fn remove_trailing_zeros(input: &str, dec: usize, prec: (u32, u32)) -> String
                 .trim_end_matches('0')
                 .trim_end_matches('.')
                 .to_string();
-            return if s.is_empty() { "0".to_string() } else { s };
+            return if s.is_empty()
+            {
+                "0".to_string()
+            }
+            else if s == "-"
+            {
+                "-0".to_string()
+            }
+            else
+            {
+                s
+            };
         }
     };
     let dec = if dec == usize::MAX - 1
@@ -1753,7 +1935,7 @@ fn remove_trailing_zeros(input: &str, dec: usize, prec: (u32, u32)) -> String
             num.insert(dec, '.');
             num = Float::parse(num)
                 .unwrap()
-                .complete(prec.0)
+                .complete(prec)
                 .to_integer()
                 .unwrap()
                 .to_string();
@@ -1766,3 +1948,24 @@ fn remove_trailing_zeros(input: &str, dec: usize, prec: (u32, u32)) -> String
         }
     }
 }
+fn notate(options: Options, colors: &Colors) -> String
+{
+    format!(
+        "{}{}",
+        if options.color
+        {
+            colors.sci.to_string()
+        }
+        else
+        {
+            String::new()
+        },
+        match options.notation
+        {
+            SmallEngineering => "e",
+            LargeEngineering => "E",
+            Scientific => "*10^",
+            Normal => "",
+        },
+    )
+}
